package main_test

import (
	"math/rand"
	"strconv"
	"testing"
	"time"

	. "git.canopsis.net/canopsis/go-revolution/cmd/engine-axe"
	. "git.canopsis.net/canopsis/go-revolution/lib/canopsis"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/alarm"
	"github.com/globalsign/mgo/bson"
	. "github.com/smartystreets/goconvey/convey"
	"github.com/streadway/amqp"
)

type EngineTestAxe struct {
	EngineAxe
}

func (e *EngineTestAxe) ConsumerChan() (<-chan amqp.Delivery, error) {
	ch := make(chan amqp.Delivery)
	return ch, nil
}

type testSendAlarm struct {
	Alarms []string
	Status []int
}

func testNewEngineAxe(sessions *CpsSessions) (*EngineTestAxe, *testSendAlarm, error) {
	tsa := testSendAlarm{
		Alarms: make([]string, 0),
	}

	alarmService := alarm.NewService(sessions.Mongo.Copy(), nil, false)

	engine := EngineTestAxe{
		EngineAxe: EngineAxe{
			DefaultEngine:   NewDefaultEngine(time.Second*60, true, true, sessions.AmqpSub),
			Sessions:        sessions,
			PeriodicalMongo: sessions.Mongo.Copy(),
			AlarmService:    alarmService,
		},
	}
	_, err := engine.ConsumerChan()
	return &engine, &tsa, err
}

// Cleaning-up entity collection to evaluate insert/update perfs
func cleanUp(sessions *CpsSessions) {
	alarmService := alarm.NewService(sessions.Mongo.Copy(), nil, false)
	alarmService.Adapter.Remove(bson.M{})
}

func TestInitializeAxe(t *testing.T) {
	Convey("Setup axe", t, func() {
		sessions, err := NewCpsSessions(CpsSessionsConf{
			Mongo:   true,
			AmqpSub: true,
			Redis:   true,
		})
		So(err, ShouldBeNil)
		defer sessions.Close()
		So(sessions.Mongo, ShouldNotBeNil)
		So(sessions.AmqpSub, ShouldNotBeNil)
		So(sessions.Redis, ShouldNotBeNil)
		sessions.Redis.FlushDB()

		engine, _, err := testNewEngineAxe(sessions)
		So(err, ShouldBeNil)

		Convey("Initialize will set correct warmups", func() {
			err := engine.Initialize()
			So(err, ShouldBeNil)

			r := sessions.Redis.Exists("connector:c1")
			So(r.Err(), ShouldBeNil)

			rttl := sessions.Redis.TTL("connector:c1")
			So(rttl.Err(), ShouldBeNil)
		})
	})
}

func BenchmarkWorkAxe(b *testing.B) {
	sessions, err := NewCpsSessions(CpsSessionsConf{
		Mongo:   true,
		AmqpPub: true,
		AmqpSub: true,
		Redis:   true,
	})
	if err != nil {
		b.Error(err)
	}
	defer sessions.Close()
	sessions.Redis.FlushDB()
	engine, _, err := testNewEngineAxe(sessions)
	if err != nil {
		b.Error(err)
	}

	engine.Initialize()
	cleanUp(sessions)

	conn := 1
	comp := 1
	for i := 1; i < b.N; i++ {
		if i%10 == 0 {
			if i%100 == 0 {
				conn = conn + 1
			} else {
				comp = comp + 1
			}
		}

		sevent := `{
            "current_alarm": null,
            "current_entity": null,
            "timestamp": ` + strconv.Itoa(int(time.Now().Unix())) + `,
            "event_type": "check",
            "state_type": 1,
            "source_type": "resource",
            "connector": "perceval_` + strconv.Itoa(conn) + `",
            "connector_name": "pellinor",
            "component": "gallois_` + strconv.Itoa(comp) + `",
            "resource": "benchmark_work_axe_` + strconv.Itoa(i) + `",
            "state": ` + strconv.Itoa(rand.Intn(3)) + `,
            "output": "Incandescente, c'est : qui peut accaparer des objets sans ressurgir sur autrui"
        }`

		engine.WorkerProcess(amqp.Delivery{
			Body: []byte(sevent),
		})
	}
}
