package main

import (
	"flag"
	"log"
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/alarm"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/config"
	cpsprof "git.canopsis.net/canopsis/go-revolution/lib/canopsis/debug/pprof"
	"git.canopsis.net/canopsis/go-revolution/lib/utils"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis"
)

func main() {
	flagDebug := flag.Bool("d", false, "debug")
	flagPrintEventOnError := flag.Bool("printEventOnError", false, "Print event on processing error")
	flagVersion := flag.Bool("version", false, "version infos")
	flag.Parse()

	if *flagVersion {
		canopsis.PrintVersionExit()
	}

	cpsprof.StartGlobalTrace()
	defer cpsprof.StopGlobalTrace()

	sessions, err := canopsis.NewCpsSessions(canopsis.CpsSessionsConf{
		Mongo:   true,
		AmqpSub: true,
		Redis:   true,
	})
	utils.ExitOnError(err, "sessions error")
	defer sessions.Close()

	_, err = sessions.AmqpSub.QueueDeclare(
		canopsis.AxeQueueName,
		true,  // durable
		false, // autodelete
		false, // exclusive
		false, // nowait
		nil,   // args
	)
	utils.FailOnError(err, "axe queue declare")
	err = sessions.AmqpSub.QueueBind(
		canopsis.AxeQueueName,
		"Engine_axe",
		"amq.direct",
		false,
		nil,
	)
	utils.FailOnError(err, "axe queue bind")

	cfg := config.GetConf()
	err = sessions.AmqpSub.Qos(cfg.Global.PrefetchCount, cfg.Global.PrefetchSize, true)
	utils.FailOnError(err, "failed to set QoS on sub channel")

	defeng := canopsis.NewDefaultEngine(
		time.Second*60,
		true,
		true,
		sessions.AmqpSub,
	)

	defeng.Debug = *flagDebug

	engine := EngineAxe{
		DefaultEngine:     defeng,
		Sessions:          sessions,
		PeriodicalMongo:   sessions.Mongo.Copy(),
		AlarmService:      alarm.NewService(sessions.Mongo.Copy(), nil, cfg.Alarm.EnableLastEventDate),
		PrintEventOnError: *flagPrintEventOnError,
	}

	if err = canopsis.StartEngine(&engine, nil); err != nil {
		log.Fatal(err)
	}
}
