package main

import (
	"flag"
	"log"

	cpsprof "git.canopsis.net/canopsis/go-revolution/lib/canopsis/debug/pprof"
	"git.canopsis.net/canopsis/go-revolution/lib/utils"

	cps "git.canopsis.net/canopsis/go-revolution/lib/canopsis"
)

func main() {
	flagProcessEvent := flag.Bool("processEvent", true, "enable event processing. enabled by default.")
	flagCreateContext := flag.Bool("createContext", true, "enable context graph creation. enabled by default. WARNING: disable the old context-graph engine when using this.")
	flagEnrichContext := flag.Bool("enrichContext", false, "enable context graph enrichment from event. disabled by default. WARNING: disable the old context-graph engine when using this.")
	flagPublishQueue := flag.String("publishQueue", "Engine_event_filter", "Publish event to this queue.")
	flagExcludeEnrich := flag.String("enrichExclude", "", "Coma separated list of fields that shall not be part of context enrichment.")
	flagIncludeEnrich := flag.String("enrichInclude", "", "Coma separated list of the only fields that will be part of context enrichment. If present, -enrichExclude is ignored.")
	flagDebug := flag.Bool("d", false, "debug")
	flagPrintEventOnError := flag.Bool("printEventOnError", false, "Print event on processing error")
	flagVersion := flag.Bool("version", false, "version infos")
	flagPurge := flag.Bool("purge", false, "purge consumer queue(s) before work")
	flag.Parse()

	if *flagVersion {
		cps.PrintVersionExit()
	}

	cpsprof.StartGlobalTrace()
	defer cpsprof.StopGlobalTrace()

	sessions, err := cps.NewCpsSessions(cps.CpsSessionsConf{
		Mongo:   true,
		AmqpSub: true,
		AmqpPub: true,
		Redis:   true,
	})
	utils.ExitOnError(err, "sessions error")
	defer sessions.Close()

	engine, err := NewEngineChe(
		*flagProcessEvent,
		*flagCreateContext,
		*flagEnrichContext,
		*flagPublishQueue,
		*flagExcludeEnrich,
		*flagIncludeEnrich,
		*flagPurge,
		sessions,
		*flagDebug,
		*flagPrintEventOnError,
	)
	if err != nil {
		log.Fatalf("new engine: %v", err)
	}

	if *flagProcessEvent {
		log.Printf("Event processing ENABLED")
	} else {
		log.Printf("Event processing DISABLED")
	}

	if *flagCreateContext {
		log.Printf("Context creation ENABLED")
	} else {
		log.Printf("Context creation DISABLED")
	}

	if *flagEnrichContext {
		log.Printf("Context enrichment ENABLED")
	} else {
		log.Printf("Context enrichment DISABLED")
	}

	if err = cps.StartEngine(engine, nil); err != nil {
		log.Fatalf("start engine: %v", err)
	}
}
