package main

import (
	"flag"
	"log"
	"time"

	cps "git.canopsis.net/canopsis/go-revolution/lib/canopsis"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/config"
	cpsprof "git.canopsis.net/canopsis/go-revolution/lib/canopsis/debug/pprof"
	"git.canopsis.net/canopsis/go-revolution/lib/utils"
)

func main() {
	flagVersion := flag.Bool("version", false, "version infos")
	flag.Parse()

	if *flagVersion {
		cps.PrintVersionExit()
	}

	cpsprof.StartGlobalTrace()
	defer cpsprof.StopGlobalTrace()

	scfg := cps.CpsSessionsConf{
		Redis:   true,
		AmqpPub: true,
		AmqpSub: true,
	}
	sessions, err := cps.NewCpsSessions(scfg)
	utils.ExitOnError(err, "sessions error")
	defer sessions.Close()

	cfg := config.GetConf()
	sessions.AmqpSub.Qos(cfg.Global.PrefetchCount, cfg.Global.PrefetchSize, true)
	engine := EngineHeartBeat{
		DefaultEngine: cps.NewDefaultEngine(time.Second*60, true, true, sessions.AmqpSub),
		Pub:           sessions.AmqpPub,
		Sessions:      sessions,
		Cfg:           cfg,
	}

	engine.SendAlarmFunc = engine.sendalarm

	log.Print("starting heartbeat")
	err = cps.StartEngine(&engine, nil)
	if err != nil {
		log.Print(err)
	}
}
