package main_test

import (
	"testing"
	"time"

	"github.com/streadway/amqp"

	. "git.canopsis.net/canopsis/go-revolution/cmd/engine-stat"
	. "git.canopsis.net/canopsis/go-revolution/lib/canopsis"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/types"
	. "github.com/smartystreets/goconvey/convey"
)

type testSendAlarm struct {
	Alarms []string
	Status []int
}

type TestEngineStat struct {
	EngineStat
}

func (e *TestEngineStat) ConsumerChan() (<-chan amqp.Delivery, error) {
	return make(chan amqp.Delivery), nil
}

func testNewEngineStat(sessions *CpsSessions) (*TestEngineStat, *testSendAlarm, error) {
	tsa := testSendAlarm{
		Alarms: make([]string, 0),
	}
	engine := TestEngineStat{
		EngineStat: EngineStat{
			DefaultEngine: NewDefaultEngine(time.Second*60, true, true, sessions.AmqpSub),
			Sessions:      sessions,
		},
	}
	_, err := engine.ConsumerChan()
	return &engine, &tsa, err
}

func TestInitializeHeartBeat(t *testing.T) {
	Convey("Given a setup", t, func() {
		sessions, err := NewCpsSessions(CpsSessionsConf{Influx: true, AmqpSub: true})
		So(err, ShouldBeNil)
		defer sessions.Close()
		So(sessions.Amqp, ShouldNotBeNil)
		So(sessions.AmqpSub, ShouldNotBeNil)
		So(sessions.Influx, ShouldNotBeNil)

		engine, _, err := testNewEngineStat(sessions)

		So(err, ShouldBeNil)

		Convey("Initialize will set correct warmups", func() {
			err := engine.Initialize()
			So(err, ShouldBeNil)
		})
	})
}

func TestWorkStat(t *testing.T) {
	Convey("Given a setup", t, func() {
		sessions, err := NewCpsSessions(CpsSessionsConf{Influx: true, AmqpSub: true})
		So(err, ShouldBeNil)
		defer sessions.Close()
		So(sessions.Amqp, ShouldNotBeNil)
		So(sessions.AmqpSub, ShouldNotBeNil)
		So(sessions.Influx, ShouldNotBeNil)

		engine, _, err := testNewEngineStat(sessions)
		So(err, ShouldBeNil)

		Convey("And given a correct event", func() {
			var event = types.Event{
				Connector:     "non",
				ConnectorName: "ce",
				Component:     "netaitpas",
				Resource:      "leradot",
				EventType:     types.EventTypeCheck,
				SourceType:    types.SourceTypeResource,
				State:         types.AlarmStateOK,
				Output:        "de la méduse ce bateau ; qu'on se le dise",
			}

			Convey("Then WorkStat should generate a Ok StatItem", func() {
				item := engine.WorkStat(event)
				So(item, ShouldNotBeNil)
				So(item.Resource, ShouldEqual, "leradot")
				So(item.EventOk, ShouldEqual, 1)
				So(item.EventKo, ShouldEqual, 0)
			})

			Convey("Then WorkStat should genereate a Ko StatItem", func() {
				event.State = types.AlarmStateMajor
				item := engine.WorkStat(event)
				So(item, ShouldNotBeNil)
				So(item.EventOk, ShouldEqual, 0)
				So(item.EventKo, ShouldEqual, 1)
			})
		})

		Convey("And given a bad event", func() {
			var event = types.Event{
				Connector:     "donnes",
				ConnectorName: "moi",
				Component:     "ta",
				Resource:      "main",
				EventType:     types.EventTypeAssocTicket,
				SourceType:    types.SourceTypeResource,
				Output:        "et prends la mienne",
			}

			Convey("Then WorkStat should not genereate a StatItem", func() {
				item := engine.WorkStat(event)
				So(item, ShouldBeNil)
			})
		})
	})
}
