package main

import (
	"flag"
	"log"
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/config"
	cpsprof "git.canopsis.net/canopsis/go-revolution/lib/canopsis/debug/pprof"
	"git.canopsis.net/canopsis/go-revolution/lib/utils"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis"
)

func main() {
	flagDebug := flag.Bool("d", false, "debug")
	flagVersion := flag.Bool("version", false, "version infos")
	flag.Parse()

	if *flagVersion {
		canopsis.PrintVersionExit()
	}

	cpsprof.StartGlobalTrace()
	defer cpsprof.StopGlobalTrace()

	scfg := canopsis.CpsSessionsConf{
		AmqpSub: true,
		Influx:  true,
		Mongo:   true,
	}
	sessions, err := canopsis.NewCpsSessions(scfg)
	utils.ExitOnError(err, "sessions error")
	defer sessions.Close()

	cfg := config.GetConf()
	sessions.AmqpSub.Qos(cfg.Global.PrefetchCount, cfg.Global.PrefetchSize, true)

	defeng := canopsis.NewDefaultEngine(
		time.Second*60,
		true,
		true,
		sessions.AmqpSub,
	)
	defeng.Debug = *flagDebug

	engine := EngineStat{
		DefaultEngine: defeng,
		Sessions:      sessions,
		Cfg:           cfg,
	}

	log.Print("starting stats")
	err = canopsis.StartEngine(&engine, nil)
	if err != nil {
		log.Print(err)
	}
}
