package amqp

import (
	"fmt"
	"os"

	"git.canopsis.net/canopsis/go-revolution/lib/utils"
	amq "github.com/streadway/amqp"
)

// Environment variables linked to session parameters
const (
	EnvURL = "CPS_AMQP_URL"
)

// NewSession creates a new connection to an AMQP bus,
// using env var EnvCpsAmqpURL as configuration.
func NewSession() (*amq.Connection, error) {
	url := os.Getenv(EnvURL)
	if url == "" {
		return nil, fmt.Errorf("environment variable %s empty", EnvURL)
	}
	return amq.Dial(url)
}

// NewSessionFail calls NewSession and utils.ExitOnError
func NewSessionFail() *amq.Connection {
	s, err := NewSession()
	utils.ExitOnError(err, "amqp session")
	return s
}
