package cache

// Cacheable makes your struct cache-able
type Cacheable interface {
	CacheID() string
}

// Cache unifies cache usage across the project.
type Cache interface {
	// GetLocal returns the object in local cache, if it exists.
	// Returns the object, true if found
	//   nil, false if not found
	Get(id string) (interface{}, bool)

	// Set both remote and local caches
	Set(element Cacheable) error

	// Drop both remote and local cache entry
	Drop(ids ...string) error

	// Flush removes all entries in remote and local cache
	Flush() error
}
