package cache

type kvCache struct {
	cache map[string]interface{}
}

// NewKV is a local Key: Value storage using a stock golang map[string]interface{}.
// Very useful for tests but could be used for any local cache.
func NewKV() Cache {
	c := kvCache{
		cache: make(map[string]interface{}),
	}
	return &c
}

func (c *kvCache) Get(id string) (interface{}, bool) {
	value, exists := c.cache[id]
	return value, exists
}

func (c *kvCache) Set(element Cacheable) error {
	c.cache[element.CacheID()] = element
	return nil
}

func (c *kvCache) Drop(ids ...string) error {
	for _, id := range ids {
		delete(c.cache, id)
	}
	return nil
}

func (c *kvCache) Flush() error {
	c.cache = make(map[string]interface{})
	return nil
}
