package cache_test

import (
	"testing"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/cache"
	"git.canopsis.net/canopsis/go-revolution/lib/testutils"
	. "github.com/smartystreets/goconvey/convey"
)

type TestCacheVal int

func (t TestCacheVal) CacheID() string {
	return "exists"
}

func NewTestCache() cache.Cache {
	kv := cache.NewKV()
	return cache.NewThreadSafeCache(kv)
}

func TestThreadSafeCache(t *testing.T) {
	Convey("Setup", t, func() {
		tsc := NewTestCache()

		v := TestCacheVal(42)
		err := tsc.Set(v)
		So(err, ShouldBeNil)

		cv, exists := tsc.Get("exists")
		So(int(cv.(TestCacheVal)), ShouldEqual, 42)
		So(exists, ShouldBeTrue)

		cv, exists = tsc.Get("noexists")
		So(exists, ShouldBeFalse)

	})
}

func TestCacheConcurrency(t *testing.T) {
	testutils.SkipLongIfSet(t)
	Convey("Setup", t, func() {
		tsc := NewTestCache()

		cacheWork := func() {
			run := 30000000

			tcv := TestCacheVal(1)
			tsc.Set(tcv)

			finished := make(chan bool, 2)

			cacheRead := func() {
				for i := 1; i < run; i++ {
					val, exists := tsc.Get(tcv.CacheID())
					tval, ok := val.(TestCacheVal)
					if (ok && int(tval) == 0) || exists == false || !ok {
						t.Fatal("got 0 value")
					}
				}
				finished <- true
			}

			cacheWrite := func() {
				for i := 1; i < run; i++ {
					tsc.Set(TestCacheVal(i))
				}
				finished <- true
			}

			go cacheRead()
			go cacheWrite()
			<-finished
			<-finished
		}

		So(cacheWork, ShouldNotPanic)

	})
}
