package context_test

import (
	"testing"
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/context"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/types"
	. "github.com/smartystreets/goconvey/convey"
)

func getEventResource() types.Event {
	e := types.Event{
		Connector:     "conn1",
		ConnectorName: "conn1",
		Component:     "comp1",
		Resource:      "res1",
		SourceType:    types.SourceTypeResource,
	}
	e.Format()

	return e
}

func TestBuilder(t *testing.T) {
	Convey("Setup", t, func() {
		tea := newTestEntityAdapter()
		ec := context.NewBuilder(tea)
		e := getEventResource()
		eventEntity := ec.Handle(e)

		So(eventEntity, ShouldNotBeNil)
		So(eventEntity.Type, ShouldEqual, e.SourceType)

		res, updates := ec.Extract(time.Now())

		So(len(updates), ShouldEqual, 0)

		So(res, ShouldContainKey, "conn1/conn1")
		So(res, ShouldContainKey, "res1/comp1")
		So(res, ShouldContainKey, "comp1")

		So(res["conn1/conn1"].Entity.Type, ShouldEqual, types.EntityTypeConnector)
		So(res["res1/comp1"].Entity.Type, ShouldEqual, types.EntityTypeResource)
		So(res["comp1"].Entity.Type, ShouldEqual, types.EntityTypeComponent)

		So(res["conn1/conn1"].Entity.Impacts.List(), ShouldContain, "res1/comp1")
		So(res["res1/comp1"].Entity.Impacts.List(), ShouldContain, "comp1")
		So(res["comp1"].Entity.Impacts.List(), ShouldContain, "conn1/conn1")

		So(res["conn1/conn1"].Entity.Depends.List(), ShouldContain, "comp1")
		So(res["res1/comp1"].Entity.Depends.List(), ShouldContain, "conn1/conn1")
		So(res["comp1"].Entity.Depends.List(), ShouldContain, "res1/comp1")
	})
}

func BenchmarkBuilder(b *testing.B) {
	tea := newTestEntityAdapter()
	ec := context.NewBuilder(tea)
	e := getEventResource()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		ec.Handle(e)
	}
}
