package context

import (
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/types"
)

// EnrichmentCenter is the high level API for context management.
type EnrichmentCenter interface {
	// Handle context creation and update on received event.
	// param source: the original event as byte slice.
	// param ef: which fields are included/excluded from extra infos enrichment.
	//     see EnrichFields doc for more infos.
	Handle(event types.Event, ef EnrichFields) *types.Entity

	// Flush documents that did not reiceived any updates after that time.
	Flush(timeFilter time.Time)
}

// Builder is the low level API for context management.
type Builder interface {
	Handle(event types.Event) *types.Entity
	Enrich(event types.Event, ef EnrichFields) *types.Entity
	Create(entity types.Entity) EntityState
	Extract(timeFilter time.Time) (map[string]EntityState, map[string]EntityState)
	// number of objects ready to be flushed since they were not updated after a given time
	Old(timeFilter time.Time) uint64
}
