package context

import (
	"sync"
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/types"
)

type safeEC struct {
	unsafe EnrichmentCenter
	l      sync.RWMutex
}

func (s *safeEC) Flush(timeFilter time.Time) {
	s.l.Lock()
	defer s.l.Unlock()
	s.unsafe.Flush(timeFilter)
}

func (s *safeEC) Handle(event types.Event, ef EnrichFields) *types.Entity {
	s.l.Lock()
	defer s.l.Unlock()
	return s.unsafe.Handle(event, ef)
}

// NewSafeEnrichmentCenter wrap EnrichmentCenter with a sync.RWMutex
func NewSafeEnrichmentCenter(ec EnrichmentCenter) EnrichmentCenter {
	sec := safeEC{
		unsafe: ec,
		l:      sync.RWMutex{},
	}

	return &sec
}
