package canopsis

import "strings"

// Error types
const (
	ErrorTypeMongoConnection = iota
	ErrorTypeRedisConnectionClosed
	ErrorTypeAMQPConnection
	ErrorTypeNetworkConnectionRefused
	ErrorTypeEOF     // connection closed unexpectedly, end of file...
	ErrorTypeNoError // nil
	ErrorTypeUnknown
)

// ErrorType returns on of ErrorType* constant depending on err.Error() content.
func ErrorType(err error) int {
	if err == nil {
		return ErrorTypeNoError
	}
	if err.Error() == "EOF" {
		return ErrorTypeEOF
	}
	if err.Error() == "redis: client is closed" {
		return ErrorTypeRedisConnectionClosed
	}
	if strings.Contains(err.Error(), "getsockopt: connection refused") {
		return ErrorTypeNetworkConnectionRefused
	}

	return ErrorTypeUnknown
}
