package event

import (
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/alarm"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/entity"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/types"
	"git.canopsis.net/canopsis/go-revolution/lib/errt"
)

// Adapter regroups AlarmDBAdapter and EntityDBAdapter
type Adapter struct {
	AlarmService  *alarm.Service
	EntityAdapter *entity.DBAdapter
	entityCache   map[string]types.Entity
}

// EnrichReport is the status of an enrichment
type EnrichReport struct {
	HasEntity bool
	Error     error
}

// NewEventDBAdapter give the correct event adapter
func NewEventDBAdapter(alarmService *alarm.Service, entityAdapter *entity.DBAdapter) Adapter {
	return Adapter{
		AlarmService:  alarmService,
		EntityAdapter: entityAdapter,
		entityCache:   make(map[string]types.Entity),
	}
}

// Enrich the event with alarm and entity
func (e *Adapter) Enrich(event types.Event) (types.Event, EnrichReport) {
	var report EnrichReport
	eid := event.GetEID()

	entity, err := e.EntityAdapter.GetEntityByID(eid)
	if err == nil {
		report.Error = nil
		report.HasEntity = true
		event.Entity = &entity
	} else {
		_, ok := err.(errt.NotFound)
		if ok {
			report.Error = nil
			report.HasEntity = false
		} else {
			report.Error = err
			report.HasEntity = false

			// exit early because the error is unknown
			return event, report
		}
	}

	return event, report
}
