package canopsis

import (
	"log"
	"os"
	"strconv"
	"time"
)

// Retry parameters
const (
	DefaultMaxRetry         = 10
	DefaultRetryDelay       = 30
	DefaultWaitFirstAttempt = 10
)

func getValue(paramName string, defaultValue int) int {
	var value string
	var res int
	var err error

	value = os.Getenv(EnvCpsStartMaxRetry)

	if value == "" {
		return defaultValue
	}

	res, err = strconv.Atoi(value)
	if err != nil {
		log.Fatalf("graceful start: getValue: %s=%v: %v", paramName, value, err)
	}
	return res
}

// GracefullStart will try to initialize every
func GracefullStart(conf CpsSessionsConf) (*CpsSessions, error) {
	var err error
	var maxRetry int
	var retryDelay int
	var waitFirstAttempt int
	var attempt int

	maxRetry = getValue(EnvCpsStartMaxRetry, DefaultMaxRetry)
	retryDelay = getValue(EnvCpsStartRetryDelay, DefaultRetryDelay)
	waitFirstAttempt = getValue(EnvCpsStartWaitFirst, DefaultWaitFirstAttempt)

	log.Printf("Gracefull start : waiting %ds\n", waitFirstAttempt)

	time.Sleep(time.Duration(waitFirstAttempt) * time.Second)

	attempt = 1
	log.Printf("Gracefull start : attempt %d.\n", attempt)
	sessions, err := NewCpsSessions(conf)
	for err != nil && attempt < maxRetry {
		time.Sleep(time.Duration(retryDelay) * time.Second)
		attempt = attempt + 1
		log.Printf("Gracefull start : attempt %d.\n", attempt)
		sessions, err = NewCpsSessions(conf)
	}

	if err != nil {
		log.Print("Gracefull start : Failed to connect to at least one service.\n")
		return nil, err
	}
	log.Print("Gracefull start : Succeed.\n")
	return sessions, nil
}
