package heartbeat

import (
	"errors"
	"strings"
	"time"
)

// Item contains the required informations for the HeartBeat engine
// to work.
type Item struct {
	Mappings    []map[string]string `toml:"mappings"`
	MaxDuration time.Duration       `toml:"maxduration"`
	Registered  map[string]int      `toml:"registered"`
}

// NewItem creates a HeartBeatItem instance for you.
func NewItem(maxDuration time.Duration) Item {
	return Item{
		Mappings:    make([]map[string]string, 0),
		MaxDuration: maxDuration,
		Registered:  make(map[string]int),
	}
}

// ID returns as string the full identifier to compare with
// IEventID fo the HeartBeat engine to work.
func (li *Item) ID() string {
	idvalues := make([]string, 0)
	for _, mapping := range li.Mappings {
		for field, matches := range mapping {
			idvalues = append(idvalues, field+":"+matches)
			break
		}
	}
	return strings.Join(idvalues, ".")
}

// AddMapping simplifies your life to declare a useful HeartBeatItem.
// You can safely ignore the error, it only says that the mapping
// you try to insert already exists.
//
// The Registered in which you add mappings is important to build the final
// id with HeartBeatItem.ID()
func (li *Item) AddMapping(field, matches string) error {
	if _, ok := li.Registered[field]; ok {
		return errors.New("mapping already exists")
	}
	li.Mappings = append(li.Mappings, map[string]string{field: matches})
	li.Registered[field] = len(li.Mappings) - 1
	return nil
}
