package heartbeat_test

import (
	"testing"

	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/heartbeat"
	"git.canopsis.net/canopsis/go-revolution/lib/canopsis/types"
)

func TestHeartBeatMappingsID(t *testing.T) {
	li := heartbeat.NewItem(0)
	li.AddMapping("connector", "test")
	li.AddMapping("component", "testc")

	id := li.ID()
	if id != "connector:test.component:testc" {
		t.Fatalf("bad id: %s", id)
	}
}

func TestHeartBeatMappingsDup(t *testing.T) {
	li := heartbeat.NewItem(0)
	err1 := li.AddMapping("connector", "test")
	err2 := li.AddMapping("connector", "test")

	if err1 != nil {
		t.Fatalf("should not raise error: %v", err1)
	}

	if err2 == nil {
		t.Fatalf("should raise an error")
	}

	if len(li.Mappings) != 1 {
		t.Fatalf("bad mappings lenght: %d", len(li.Mappings))
	}
}

func BenchmarkHeartBeatItemID(b *testing.B) {
	// We will always have heartBeatItems in memory
	li := heartbeat.NewItem(0)
	li.AddMapping("connector", "test")
	li.AddMapping("connector_name", "test")

	for i := 0; i < b.N; i++ {
		sevent := `{
		"connector": "test",
		"connector_name": "test",
		"source_type": "resource",
		"component": "localhost",
		"resource": "benchmarkheartBeatItemid",
		"event_type": "check",
		"status": 0,
		"output": "Mieux, mieux : Un tiers chou, un tiers tulipe, et un tiers blette !"
	}`

		var ievent types.GenericEvent
		err := ievent.JSONUnmarshal([]byte(sevent))
		if err != nil {
			b.Fatalf("json error: %v", err)
		}
		eid, err := ievent.PartialID(li)
		if err != nil {
			b.Fatalf("ieventid: %v", err)
		}

		id := li.ID()
		if id != eid {
			b.Fatalf("bad id: %s", id)
		}
	}
}
