package canopsis_test

import (
	"testing"

	cps "git.canopsis.net/canopsis/go-revolution/lib/canopsis"
	"git.canopsis.net/canopsis/go-revolution/lib/utils"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCpsSessions(t *testing.T) {
	Convey("Testing different cps sessions configurations", t, func() {
		Convey("Testing AmqpPub only", func() {
			cfg := cps.CpsSessionsConf{
				AmqpPub: true,
			}
			sessions, err := cps.NewCpsSessions(cfg)
			So(err, ShouldBeNil)
			So(sessions, ShouldNotBeNil)
			So(sessions.Amqp, ShouldNotBeNil)
			So(sessions.AmqpPub, ShouldNotBeNil)
			So(sessions.AmqpSub, ShouldBeNil)
			So(sessions.Mongo, ShouldBeNil)
			So(sessions.Redis, ShouldBeNil)
			So(sessions.Influx, ShouldBeNil)
			Convey("Closing sessions", func() {
				sessions.Close()
				So(sessions.Amqp, ShouldBeNil)
				So(sessions.AmqpPub, ShouldBeNil)
				So(sessions.AmqpSub, ShouldBeNil)
			})
		})

		Convey("Testing all sessions", func() {
			cfg := cps.CpsSessionsConf{
				Mongo:   true,
				Redis:   true,
				AmqpPub: true,
				AmqpSub: true,
				Influx:  true,
			}
			sessions, err := cps.NewCpsSessions(cfg)
			So(err, ShouldBeNil)
			So(sessions, ShouldNotBeNil)
			So(sessions.Amqp, ShouldNotBeNil)
			So(sessions.AmqpPub, ShouldNotBeNil)
			So(sessions.AmqpSub, ShouldNotBeNil)
			So(sessions.Mongo, ShouldNotBeNil)
			So(sessions.Redis, ShouldNotBeNil)
			So(sessions.Influx, ShouldNotBeNil)
			Convey("Closing all sessions", func() {
				sessions.Close()
				So(sessions.Amqp, ShouldBeNil)
				So(sessions.AmqpPub, ShouldBeNil)
				So(sessions.AmqpSub, ShouldBeNil)
				So(sessions.Mongo, ShouldBeNil)
				So(sessions.Redis, ShouldBeNil)
				So(sessions.Influx, ShouldBeNil)
			})
		})
	})
}

func ExampleNewCpsSessions() {
	cfg := cps.CpsSessionsConf{
		Mongo:   true,
		AmqpSub: true,
	}
	sessions, err := cps.NewCpsSessions(cfg)
	utils.FailOnError(err, "session error")
	defer sessions.Close()
	sessions.Mongo.DB("canopsis").C("mycoll").Find(nil)
}
