package types

import (
	"time"
)

// Entity types
const (
	EntityTypeConnector = "connector"
	EntityTypeComponent = "component"
	EntityTypeResource  = "resource"
)

// Measurement ...
/*
type Measurement struct {
	ID   string   `bson:"_id" json:"_id"`
	tags []string `bson:"tags" json:"tags"`
}
*/

// Info contain extra values for the entity
type Info struct {
	Name        string `bson:"name" json:"name"`
	Description string `bson:"description" json:"description"`
	Value       string `bson:"value" json:"value"`
}

// Entity ...
type Entity struct {
	ID            string    `bson:"_id" json:"_id"`
	Name          string    `bson:"name" json:"name"`
	Impacts       DedupList `bson:"impact" json:"impact"`
	Depends       DedupList `bson:"depends" json:"depends"`
	EnableHistory []CpsTime `bson:"enable_history" json:"enable_history"`
	//Measurements  *[]Measurement `bson:"measurements" json:"measurements"`
	Measurements interface{}     `bson:"measurements" json:"measurements"` // unused collection ids
	Enabled      bool            `bson:"enabled" json:"enabled"`
	Infos        map[string]Info `bson:"infos" json:"infos"`
	Type         string          `bson:"type" json:"type"`
}

// EnsureInitialized verifies that all complex structs are well initialized
func (e *Entity) EnsureInitialized() {
	if e.Infos == nil {
		e.Infos = make(map[string]Info)
	}
	if e.EnableHistory == nil {
		e.EnableHistory = make([]CpsTime, 0)
	}
	e.Depends.EnsureInitialized()
	e.Impacts.EnsureInitialized()
}

// CacheID implements cache.Cache interface
func (e Entity) CacheID() string {
	return e.ID
}

// NewInfo instanciate a new info struct [sic]
func NewInfo(name string, description string, value string) Info {
	return Info{
		Name:        name,
		Description: description,
		Value:       value,
	}
}

// NewEntity instanciate a new entity struct [sic]
func NewEntity(id string, name string, entityType string, infos map[string]Info, impacts, depends DedupList) Entity {
	history := []CpsTime{CpsTime{time.Now()}}

	return Entity{
		ID:            id,
		Name:          name,
		Impacts:       impacts,
		Depends:       depends,
		EnableHistory: history,
		Measurements:  nil,
		Enabled:       true,
		Infos:         infos,
		Type:          entityType,
	}
}
