package types

// PyMap is a light wrapper around a map providing shortcuts
type PyMap map[string]interface{}

// Get a value under the given key if found, or return the def value.
// Use type assertion to return the concrete type.
func (m PyMap) Get(key string, def interface{}) interface{} {
	val, exists := m[key]
	if !exists {
		return def
	}

	return val
}

// Del a key from the map
func (m PyMap) Del(key string) {
	delete(m, key)
}

// Set a key to value val in the map
func (m PyMap) Set(key string, val interface{}) {
	m[key] = val
}

// NewPyMap creates a new PyMap
func NewPyMap() PyMap {
	return make(map[string]interface{})
}
