package engine

import (
	"log"
	"time"
)

// DefaultPartialPeriodical implements basic functions and lets you implement
// the actual periodical process function.
type DefaultPartialPeriodical struct {
	pm Main
}

// NewPartialPeriodical ...
func NewPartialPeriodical(m Main) PartialPeriodical {
	p := DefaultPartialPeriodical{
		pm: m,
	}
	return &p
}

// PeriodicalContinue returns the internal engine continue state
func (e *DefaultPartialPeriodical) PeriodicalContinue() bool {
	return e.pm.Continue()
}

// PeriodicalWait returns 60s by default
func (e *DefaultPartialPeriodical) PeriodicalWait() time.Duration {
	return time.Second * 60
}

// PeriodicalStopped acknowledge the periodical process stop
func (e *DefaultPartialPeriodical) PeriodicalStopped() {
	log.Printf("periodical stopped successfuly")
	e.pm.ProcessStopped()
	e.pm.AskStop(ExitOK)
}

// PeriodicalRecover a panic from the Periodical Process and ask the engine to stop
func (e *DefaultPartialPeriodical) PeriodicalRecover() {
	if r := recover(); r != nil {
		log.Printf("periodical recover: %v", r)
		e.pm.AskStop(ExitPanic)
		e.PeriodicalStopped()
	}
}
