package engine

import (
	"log"
)

// DefaultPartialWorker lets you implement the actual Worker Process
type DefaultPartialWorker struct {
	wm Main
}

// NewPartialWorker ...
func NewPartialWorker(m Main) PartialWorker {
	w := DefaultPartialWorker{
		wm: m,
	}

	return &w
}

// WorkerContinue returns the internal continue state
func (e *DefaultPartialWorker) WorkerContinue() bool {
	return e.wm.Continue()
}

// WorkerStopped acknowledge Worker Process stopped
func (e *DefaultPartialWorker) WorkerStopped() {
	log.Println("worker stopped successfuly")
	e.wm.ProcessStopped()
	e.wm.AskStop(ExitOK)
}

// WorkerRecover a panic from Worker Process and asks the engine to stop
func (e *DefaultPartialWorker) WorkerRecover() {
	if r := recover(); r != nil {
		log.Printf("worker recover: %v", r)
		e.wm.AskStop(ExitPanic)
		e.WorkerStopped()
	}
}
