package engine_test

import (
	"log"
	"testing"
	"time"

	"github.com/streadway/amqp"

	"git.canopsis.net/canopsis/go-revolution/lib/engine"
	. "github.com/smartystreets/goconvey/convey"
)

func TestDefaultEngine(t *testing.T) {
	Convey("Setup default engine", t, func() {
		e := engine.NewDefaultEngine()
		err := engine.Start(e)

		So(err, ShouldNotBeNil)
		So(err.Error(), ShouldEqual, "engine does not implement any of Worker or Periodical interfaces")
	})
}

type TestWorker struct {
	engine.PartialWorker
}

func (e *TestWorker) WorkerConsumer() (<-chan amqp.Delivery, error) {
	msgs := make(chan amqp.Delivery, 10)
	msgs <- amqp.Delivery{Body: []byte("coucou0")}
	msgs <- amqp.Delivery{Body: []byte("coucou1")}
	msgs <- amqp.Delivery{Body: []byte("coucou2")}
	msgs <- amqp.Delivery{Body: []byte("coucou3")}
	msgs <- amqp.Delivery{Body: []byte("coucou4")}
	return msgs, nil
}

func (e *TestWorker) WorkerProcess(msg amqp.Delivery) error {
	log.Printf("msg: %s", string(msg.Body))
	time.Sleep(time.Second)
	panic("blargh")
	return nil
}

type TestPeriodical struct {
	engine.PartialPeriodical
}

func (e *TestPeriodical) PeriodicalWait() time.Duration {
	return time.Second
}

func (e *TestPeriodical) PeriodicalProcess() error {
	log.Printf("periodical doing stuff")
	time.Sleep(time.Second * 2)
	panic("bleurp")
	return nil
}

func TestFullEngine(t *testing.T) {
	Convey("Setup full engine", t, func() {
		de := engine.NewDefaultEngine()
		we := engine.NewPartialWorker(de)
		pe := engine.NewPartialPeriodical(de)

		tw := TestWorker{
			PartialWorker: we,
		}

		tp := TestPeriodical{
			PartialPeriodical: pe,
		}

		e := engine.FullEngine{
			Main:       de,
			Worker:     &tw,
			Periodical: &tp,
		}

		err := engine.Start(e)
		So(err, ShouldBeNil)
	})
}

type TestWorkerEngine struct {
	engine.Main
	engine.Worker
}

func TestWorkerEngineOnly(t *testing.T) {
	Convey("Setup worker engine", t, func() {
		de := engine.NewDefaultEngine()
		we := engine.NewPartialWorker(de)

		tw := TestWorker{
			PartialWorker: we,
		}

		twe := TestWorkerEngine{
			Main:   de,
			Worker: &tw,
		}

		err := engine.Start(twe)
		So(err, ShouldBeNil)
	})
}

type TestPeriodicalEngine struct {
	engine.Main
	engine.Periodical
}

func TestPeriodicalEngineOnly(t *testing.T) {
	Convey("Setup periodical engine", t, func() {
		de := engine.NewDefaultEngine()
		pe := engine.NewPartialPeriodical(de)

		tp := TestPeriodical{
			PartialPeriodical: pe,
		}

		tpe := TestPeriodicalEngine{
			Main:       de,
			Periodical: &tp,
		}

		err := engine.Start(tpe)
		So(err, ShouldBeNil)
	})
}
