package mongo

import (
	"io"

	"github.com/globalsign/mgo"

	"git.canopsis.net/canopsis/go-revolution/lib/errt"
)

type mongoError struct {
	errt.ErrT
}

type MongoError interface {
	IsMongoError()
	errt.ErrT
}

func NewMongoError(err error) error {
	if err == nil {
		return nil
	}
	return mongoError{ErrT: errt.NewErrT(err)}
}

// WrapError is a helper to convert an error to an errt error.
// If the error is unknown, returns MongoError.
func WrapError(err error) error {
	if err == nil {
		return nil
	}

	if mgo.IsDup(err) {
		return errt.NewDuplicated(err)
	}

	if err == mgo.ErrNotFound {
		return errt.NewNotFound(err)
	}

	if err == io.EOF {
		return errt.NewIOError(err)
	}

	if err == io.ErrClosedPipe {
		return errt.NewIOError(err)
	}

	if err == io.ErrNoProgress {
		return errt.NewIOError(err)
	}

	if err == io.ErrShortBuffer {
		return errt.NewIOError(err)
	}

	if err == io.ErrShortWrite {
		return errt.NewIOError(err)
	}

	if err == io.ErrUnexpectedEOF {
		return errt.NewIOError(err)
	}

	return NewMongoError(err)
}
