package mongo

import (
	"fmt"
	"net/url"
	"os"
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/utils"
	mgo "github.com/globalsign/mgo"
)

// NewSession creates a new connection to the MongoDB database.
// It uses the EnvCpsMongoUrl as configuration.
func NewSession(timeout time.Duration) (*mgo.Session, error) {
	mongoURL := os.Getenv(EnvURL)
	if mongoURL == "" {
		return nil, fmt.Errorf("environment variable %s empty", EnvURL)
	}
	_, err := url.ParseRequestURI(mongoURL)
	if err != nil {
		return nil, err
	}
	session, err := mgo.DialWithTimeout(mongoURL, timeout)
	if err != nil {
		return nil, err
	}

	session.SetSafe(&mgo.Safe{W: 1, WMode: WriteMode})
	return session, nil
}

// NewSessionFail calls NewSession() but will fail calling utils.ExitOnError
func NewSessionFail(timeout time.Duration) *mgo.Session {
	s, err := NewSession(timeout)
	utils.ExitOnError(err, "mongo session")
	return s
}
