package redis_test

import (
	"fmt"
	"os"
	"strconv"
	"testing"
	"time"

	"git.canopsis.net/canopsis/go-revolution/lib/redis"
	. "github.com/smartystreets/goconvey/convey"
)

func TestNewRedisOptions(t *testing.T) {
	redisOptions, err := redis.NewOptions("redis://user:password@host:7777", 0)
	if err != nil {
		t.Fatalf("redis options error: %v", err)
	}

	if redisOptions.Password != "password" {
		t.Fatalf("redis bad password: %s", redisOptions.Password)
	}

	if redisOptions.Addr != "host:7777" {
		t.Fatalf("redis bad addr: %s", redisOptions.Addr)
	}

	if redisOptions.DB != 0 {
		t.Fatalf("redis bad database: %d", redisOptions.DB)
	}
}

func TestBadRedisOptions(t *testing.T) {
	Convey("Testing bad redis urls", t, func() {
		Convey("Bad url", func() {
			_, err := redis.NewOptions("bla://nrausitenrste,anursiet", -1)
			So(err, ShouldNotBeNil)
		})

		Convey("Bad url - judgement day", func() {
			_, err := redis.NewOptions("", -1)
			So(err, ShouldNotBeNil)
		})

		Convey("Bad url - access denied", func() {
			_, err := redis.NewOptions("redis://user@localhost/0", -1)
			So(err, ShouldBeNil)
		})

		Convey("Bad db", func() {
			_, err := redis.NewOptions("redis://localhost/bleurk", -1)
			So(err, ShouldNotBeNil)
		})

		Convey("Bad url - i'm calling you", func() {
			oldredisurl := os.Getenv(redis.EnvURL)
			os.Setenv(redis.EnvURL, "redis://anrsuitenrstau,nrasutie;;;;...")
			_, err := redis.NewSession(-1)
			os.Setenv(redis.EnvURL, oldredisurl)
			So(err, ShouldNotBeNil)
		})

		Convey("Good url, wrong database - Bagdad cafe", func() {
			oldredisurl := os.Getenv(redis.EnvURL)
			os.Setenv(redis.EnvURL, "redis://Canyouexplainwhatyouwannado/0")
			_, err := redis.NewSession(-1)
			os.Setenv(redis.EnvURL, oldredisurl)
			So(err, ShouldNotBeNil)
		})
	})
}

func TestNewRedisSession(t *testing.T) {
	_, err := redis.NewSession(redis.CacheAlarm)
	if err != nil {
		t.Fatalf("redis client error: %v", err)
	}
}

func BenchmarkRedisSpeed(b *testing.B) {
	client, _ := redis.NewSession(redis.CacheAlarm)

	for i := 1; i < b.N; i++ {
		rid := "titi_" + strconv.Itoa(i)
		if i%4 == 0 {
			err := client.Set(rid, "toto", time.Hour*4)
			if err.Err() != nil {
				fmt.Printf("error setting key %s: %v", rid, err)
			}
		}
		_ = client.Exists(rid)
	}
}
